@@ -13,7 +13,8 @@ from .settings import FILE_SKIP_COMMENTS
 
 CIMPORT_IDENTIFIERS = ("cimport ", "cimport*", "from.cimport")
 IMPORT_START_IDENTIFIERS = ("from ", "from.import", "import ", "import*") + CIMPORT_IDENTIFIERS
-COMMENT_INDICATORS = ('"""', "'''", "'", '"', "#")
+DOCSTRING_INDICATORS = ('"""', "'''")
+COMMENT_INDICATORS = DOCSTRING_INDICATORS + ("'", '"', "#")
 CODE_SORT_COMMENTS = (
     "# isort: list",
     "# isort: dict",
@@ -29,6 +30,7 @@ def process(
     input_stream: TextIO,
     output_stream: TextIO,
     extension: str = "py",
+    raise_on_skip: bool = True,
     config: Config = DEFAULT_CONFIG,
 ) -> bool:
     """Parses stream identifying sections of contiguous imports and sorting them
@@ -53,6 +55,7 @@ def process(
     next_import_section: str = ""
     next_cimports: bool = False
     in_quote: str = ""
+    was_in_quote: bool = False
     first_comment_index_start: int = -1
     first_comment_index_end: int = -1
     contains_imports: bool = False
@@ -60,6 +63,7 @@ def process(
     first_import_section: bool = True
     indent: str = ""
     isort_off: bool = False
+    skip_file: bool = False
     code_sorting: Union[bool, str] = False
     code_sorting_section: str = ""
     code_sorting_indent: str = ""
@@ -125,17 +129,22 @@ def process(
                 line_separator = "\n"
 
             if code_sorting and code_sorting_section:
-                output_stream.write(
-                    textwrap.indent(
-                        isort.literal.assignment(
-                            code_sorting_section,
-                            str(code_sorting),
-                            extension,
-                            config=_indented_config(config, indent),
-                        ),
-                        code_sorting_indent,
-                    )
+                sorted_code = textwrap.indent(
+                    isort.literal.assignment(
+                        code_sorting_section,
+                        str(code_sorting),
+                        extension,
+                        config=_indented_config(config, indent),
+                    ),
+                    code_sorting_indent,
                 )
+                made_changes = made_changes or _has_changed(
+                    before=code_sorting_section,
+                    after=sorted_code,
+                    line_separator=line_separator,
+                    ignore_whitespace=config.ignore_whitespace,
+                )
+                output_stream.write(sorted_code)
         else:
             stripped_line = line.strip()
             if stripped_line and not line_separator:
@@ -143,10 +152,25 @@ def process(
 
             for file_skip_comment in FILE_SKIP_COMMENTS:
                 if file_skip_comment in line:
-                    raise FileSkipComment("Passed in content")
+                    if raise_on_skip:
+                        raise FileSkipComment("Passed in content")
+                    isort_off = True
+                    skip_file = True
 
-            if not in_quote and stripped_line == "# isort: off":
-                isort_off = True
+            if not in_quote:
+                if stripped_line == "# isort: off":
+                    isort_off = True
+                elif stripped_line.startswith("# isort: dont-add-imports"):
+                    add_imports = []
+                elif stripped_line.startswith("# isort: dont-add-import:"):
+                    import_not_to_add = stripped_line.split("# isort: dont-add-import:", 1)[
+                        1
+                    ].strip()
+                    add_imports = [
+                        import_to_add
+                        for import_to_add in add_imports
+                        if not import_to_add == import_not_to_add
+                    ]
 
             if (
                 (index == 0 or (index in (1, 2) and not contains_imports))
@@ -154,10 +178,11 @@ def process(
                 and stripped_line not in config.section_comments
             ):
                 in_top_comment = True
-            elif in_top_comment:
-                if not line.startswith("#") or stripped_line in config.section_comments:
-                    in_top_comment = False
-                    first_comment_index_end = index - 1
+            elif in_top_comment and (
+                not line.startswith("#") or stripped_line in config.section_comments
+            ):
+                in_top_comment = False
+                first_comment_index_end = index - 1
 
             was_in_quote = bool(in_quote)
             if (not stripped_line.startswith("#") or in_quote) and '"' in line or "'" in line:
@@ -188,7 +213,7 @@ def process(
             not_imports = bool(in_quote) or was_in_quote or in_top_comment or isort_off
             if not (in_quote or was_in_quote or in_top_comment):
                 if isort_off:
-                    if stripped_line == "# isort: on":
+                    if not skip_file and stripped_line == "# isort: on":
                         isort_off = False
                 elif stripped_line.endswith("# isort: split"):
                     not_imports = True
@@ -198,17 +223,22 @@ def process(
                     not_imports = True
                 elif code_sorting:
                     if not stripped_line:
-                        output_stream.write(
-                            textwrap.indent(
-                                isort.literal.assignment(
-                                    code_sorting_section,
-                                    str(code_sorting),
-                                    extension,
-                                    config=_indented_config(config, indent),
-                                ),
-                                code_sorting_indent,
-                            )
+                        sorted_code = textwrap.indent(
+                            isort.literal.assignment(
+                                code_sorting_section,
+                                str(code_sorting),
+                                extension,
+                                config=_indented_config(config, indent),
+                            ),
+                            code_sorting_indent,
                         )
+                        made_changes = made_changes or _has_changed(
+                            before=code_sorting_section,
+                            after=sorted_code,
+                            line_separator=line_separator,
+                            ignore_whitespace=config.ignore_whitespace,
+                        )
+                        output_stream.write(sorted_code)
                         not_imports = True
                         code_sorting = False
                         code_sorting_section = ""
@@ -235,8 +265,6 @@ def process(
                 ):
                     import_section += line
                 elif stripped_line.startswith(IMPORT_START_IDENTIFIERS):
-                    contains_imports = True
-
                     new_indent = line[: -len(line.lstrip())]
                     import_statement = line
                     stripped_line = line.strip().split("#")[0]
@@ -254,28 +282,47 @@ def process(
                                 stripped_line = line.strip().split("#")[0]
                                 import_statement += line
 
-                    cimport_statement: bool = False
                     if (
-                        import_statement.lstrip().startswith(CIMPORT_IDENTIFIERS)
-                        or " cimport " in import_statement
-                        or " cimport*" in import_statement
-                        or " cimport(" in import_statement
-                        or ".cimport" in import_statement
+                        import_statement.lstrip().startswith("from")
+                        and "import" not in import_statement
                     ):
-                        cimport_statement = True
+                        line = import_statement
+                        not_imports = True
+                    else:
+                        did_contain_imports = contains_imports
+                        contains_imports = True
 
-                    if cimport_statement != cimports or (new_indent != indent and import_section):
-                        if import_section:
-                            next_cimports = cimport_statement
-                            next_import_section = import_statement
-                            import_statement = ""
-                            not_imports = True
-                            line = ""
+                        cimport_statement: bool = False
+                        if (
+                            import_statement.lstrip().startswith(CIMPORT_IDENTIFIERS)
+                            or " cimport " in import_statement
+                            or " cimport*" in import_statement
+                            or " cimport(" in import_statement
+                            or ".cimport" in import_statement
+                        ):
+                            cimport_statement = True
+
+                        if cimport_statement != cimports or (
+                            new_indent != indent
+                            and import_section
+                            and (not did_contain_imports or len(new_indent) < len(indent))
+                        ):
+                            indent = new_indent
+                            if import_section:
+                                next_cimports = cimport_statement
+                                next_import_section = import_statement
+                                import_statement = ""
+                                not_imports = True
+                                line = ""
+                            else:
+                                cimports = cimport_statement
                         else:
-                            cimports = cimport_statement
-
-                    indent = new_indent
-                    import_section += import_statement
+                            if new_indent != indent:
+                                if import_section and did_contain_imports:
+                                    import_statement = indent + import_statement.lstrip()
+                                else:
+                                    indent = new_indent
+                        import_section += import_statement
                 else:
                     not_imports = True
 
@@ -286,13 +333,15 @@ def process(
                 and (stripped_line or end_of_file)
                 and not config.append_only
                 and not in_top_comment
-                and not in_quote
+                and not was_in_quote
                 and not import_section
                 and not line.lstrip().startswith(COMMENT_INDICATORS)
+                and not (line.rstrip().endswith(DOCSTRING_INDICATORS) and "=" not in line)
             ):
-                import_section = line_separator.join(add_imports) + line_separator
+                add_line_separator = line_separator or "\n"
+                import_section = add_line_separator.join(add_imports) + add_line_separator
                 if end_of_file and index != 0:
-                    output_stream.write(line_separator)
+                    output_stream.write(add_line_separator)
                 contains_imports = True
                 add_imports = []
 
@@ -398,7 +447,7 @@ def process(
     return made_changes
 
 
-def _indented_config(config: Config, indent: str):
+def _indented_config(config: Config, indent: str) -> Config:
     if not indent:
         return config
 
@@ -407,6 +456,7 @@ def _indented_config(config: Config, indent: str):
         line_length=max(config.line_length - len(indent), 0),
         wrap_length=max(config.wrap_length - len(indent), 0),
         lines_after_imports=1,
+        import_headings=config.import_headings if config.indented_import_headings else {},
     )
 
 
@@ -416,5 +466,4 @@ def _has_changed(before: str, after: str, line_separator: str, ignore_whitespace
             remove_whitespace(before, line_separator=line_separator).strip()
             != remove_whitespace(after, line_separator=line_separator).strip()
         )
-    else:
-        return before.strip() != after.strip()
+    return before.strip() != after.strip()
