@@ -370,6 +370,23 @@ import a
 import b
 
 
+def function():
+    pass
+"""
+    )
+
+    # if file is skipped it should output unchanged.
+    main.main(
+        ["-", "--filename", "x.py", "--skip", "x.py", "--filter-files"],
+        stdin=build_input_content(),
+    )
+    out, error = capsys.readouterr()
+    assert not error
+    assert out == (
+        """
+import b
+import a
+
 def function():
     pass
 """
